package org.gcube.portlets.user.gisviewer.client;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.gcube.common.core.utils.logging.GCUBELog;
import org.gcube.portlets.user.gisviewer.client.commons.beans.DataResult;
import org.gcube.portlets.user.gisviewer.client.commons.beans.GeoserverItem;
import org.gcube.portlets.user.gisviewer.client.commons.beans.GroupInfo;
import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerConfiguration;
import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItem;
import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItemsResult;
import org.gcube.portlets.user.gisviewer.client.commons.beans.MapViewInfo;
import org.gcube.portlets.user.gisviewer.client.commons.beans.WfsTable;
import org.gcube.portlets.user.gisviewer.client.commons.utils.ClickDataInfo;
import org.gcube.portlets.user.gisviewer.client.commons.utils.URLMakers;
import org.gcube.portlets.user.gisviewer.client.datafeature.DataPanelHandler;
import org.gcube.portlets.user.gisviewer.client.datafeature.DataResultPanel;
import org.gcube.portlets.user.gisviewer.client.layerspanel.LayersPanel;
import org.gcube.portlets.user.gisviewer.client.layerspanel.LayersPanelHandler;
import org.gcube.portlets.user.gisviewer.client.openlayers.CqlFilterHandler;
import org.gcube.portlets.user.gisviewer.client.openlayers.CqlFilterPanel;
import org.gcube.portlets.user.gisviewer.client.openlayers.OpenLayersHandler;
import org.gcube.portlets.user.gisviewer.client.openlayers.OpenLayersMap;
import org.gcube.portlets.user.gisviewer.client.openlayers.ToolBarPanel;
import org.gcube.portlets.user.gisviewer.client.openlayers.ToolbarHandler;

import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.MessageBoxEvent;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.layout.FitLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.LoadEvent;
import com.google.gwt.event.dom.client.LoadHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Image;

public class GisViewerPanel extends LayoutContainer 
implements ToolbarHandler, DataPanelHandler, LayersPanelHandler, CqlFilterHandler, OpenLayersHandler {

	private static final String ALERT_HARD_SPATIAL_QUERY = "The spatial query can take long time. Do you want to continue to download it?";
	private GisViewerLayout mainPanel;
	private LayersPanel layersPanel;
	private OpenLayersMap openLayersMap;
	private ToolBarPanel toolBarPanel;
	private GisViewerServiceAsync gisViewerServiceAsync;

	private GisViewerSaveHandler gisViewerSaveHandler;
	private GisViewerParameters parameters;
	private String projection;
	private CqlFilterPanel cqlFilterPanel = new CqlFilterPanel(this);

	private ClickDataInfo lastClickDataInfo = null;


	/**
	 * 
	 */
	public GisViewerPanel() {
		this(new GisViewerParameters());
	}


	public GisViewerPanel(GisViewerParameters parameters) {
		super();
		this.parameters = parameters; 
		this.gisViewerServiceAsync = GisViewer.service;
		this.gisViewerSaveHandler = parameters.getGisViewerSaveHandler();
		this.projection = parameters.getProjection();

		init();
	}

	/**
	 * @param parameters
	 * @param gisViewerSaveHandler
	 */
	public GisViewerPanel(GisViewerParameters parameters, GisViewerSaveHandler gisViewerSaveHandler) {
		super();
		this.parameters = parameters; 
		this.parameters.setGisViewerSaveHandler(gisViewerSaveHandler);
		this.gisViewerServiceAsync = GisViewer.service;
		this.gisViewerSaveHandler = parameters.getGisViewerSaveHandler();
		this.projection = parameters.getProjection();

		init();
	}

	/**
	 * 
	 */
	private void init() {
		// create an empty layers panel
		layersPanel = new LayersPanel(this);
		createOpenLayersMap();

		// initialize the layout
		this.setLayout(new FitLayout());
		mainPanel = new GisViewerLayout(parameters.isOpenDataPanelAtStart(), this) {
			/* (non-Javadoc)
			 * @see com.extjs.gxt.ui.client.widget.LayoutContainer#onRender(com.google.gwt.user.client.Element, int)
			 */
			@Override
			protected void onRender(Element parent, int index) {
				super.onRender(parent, index);
				mainPanel.setLayersPanel(layersPanel);		
				mainPanel.setOpenLayers(openLayersMap);
			}
		};

		this.add(mainPanel);
	}

	@Override
	protected void onRender(Element parent, int pos) {
		super.onRender(parent, pos);

		// monitoring of the datapanel open/close event
		if (parameters.getDataPanelOpenHandler()!=null)
			mainPanel.startPanelOpenMonitoring();

		getTransectUrl();

		if (parameters.getOpeningGroup()!=null)
			addGroupByGroupName(parameters.getOpeningGroup());
		else if (parameters.getOpeningLayers()!=null)
			addLayersByLayerNames(parameters.getOpeningLayers());
	};

	private void waitMessage(boolean show) {
		if (show)
			this.mask(Constants.MessageLoadingLayersData);
		else
			this.unmask();
	}

	protected void getTransectUrl() {
		gisViewerServiceAsync.getTransectUrl(new AsyncCallback<String>() {

			@Override
			public void onFailure(Throwable caught) {
				MessageBox.alert("Error retrieving the transect url", caught.getMessage(), null);
			}

			@Override
			public void onSuccess(String result) {
				openLayersMap.setTransectUrl(result);
			}
		});
	}

	public void addGroupByGroupName(final String groupName) {
		waitMessage(true);
		gisViewerServiceAsync.getGroupsInfo(groupName, new AsyncCallback<LayerItemsResult>() {
			public void onSuccess(LayerItemsResult result) {
				waitMessage(false);
				if (!result.getStatus().equals(""))
					MessageBox.alert("Error", result.getStatus(), null);

				if (result.getLayerItemsSize()!=0) {
					List<LayerItem> layerItems = result.getLayerItems();
					addLayersOnMap(layerItems, false);
					showLayer(layerItems.get(layerItems.size()-1)); // show last layer
				}
			}

			public void onFailure(Throwable caught) {
				waitMessage(false);
				Window.alert("The layers doesn't exist");
			}
		});
	}

	public void addLayersByLayerNames(String ... layerNames) {
		 addLayersByLayerNames(Arrays.asList(layerNames));
	}

	public void addLayersByLayerNames(List<String> layerNames) {
		addLayersByLayerNames(layerNames, false);
	}

	public void addLayersByLayerNamesToTop(final List<String> layerNames) {
		addLayersByLayerNames(layerNames, true);
	}

	/**
	 * @param layerItems
	 */
	public void addLayersByLayerItems(List<LayerItem> layerItems) {
		addLayersByLayerItems(layerItems, false);
	}

	/**
	 * @param layerItem
	 */
	public void addLayerByLayerItem(LayerItem layerItem) {
		List<LayerItem> layerItems = new ArrayList<LayerItem>();
		layerItems.add(layerItem);
		addLayersByLayerItems(layerItems, false);
	}

	/**
	 * @param layerItems
	 */
	public void addLayersByLayerItemsToTop(List<LayerItem> layerItems) {
		addLayersByLayerItems(layerItems, true);
	}

	/**
	 * @param layerItem
	 */
	public void addLayerByLayerItemToTop(LayerItem layerItem) {
		List<LayerItem> layerItems = new ArrayList<LayerItem>();
		layerItems.add(layerItem);
		addLayersByLayerItems(layerItems, true);
	}

//	public void addLayersByLayerConfigurations(LayerConfiguration ... layersConfigurations) {
//		this.addLayersByLayerConfigurations(Arrays.asList(layersConfigurations), false);
//	}
//
//	public void addLayersByLayerConfigurationsToTop(LayerConfiguration ... layersConfigurations) {
//		this.addLayersByLayerConfigurations(Arrays.asList(layersConfigurations), true);
//	}
//
//	private void addLayersByLayerConfigurations(final List<LayerConfiguration> layerConfigurations, final boolean toTop) {
//		waitMessage(true);
//		gisViewerServiceAsync.getLayersInfoByLayerConfigurations(layerConfigurations, new AsyncCallback<LayerItemsResult>() {
//			
//			public void onSuccess(LayerItemsResult result) {
//				waitMessage(false);
//				if (!result.getStatus().equals(""))
//					MessageBox.alert("Error", result.getStatus(), null);
//
//				if (result.getLayerItemsSize()!=0)
//					addLayersOnMap(result.getLayerItems(), toTop);
//			}
//
//			public void onFailure(Throwable caught) {
//				waitMessage(false);
//				caught.printStackTrace();
//				Window.alert("The layers doesn't exist");
//			}
//		});
//	}
	
	private void addLayersByLayerNames(final List<String> layerNames, final boolean toTop) {
		waitMessage(true);
		gisViewerServiceAsync.getLayersInfo(layerNames, new AsyncCallback<LayerItemsResult>() {			
			public void onSuccess(LayerItemsResult result) {
				waitMessage(false);
				if (!result.getStatus().equals(""))
					MessageBox.alert("Error", result.getStatus(), null);

				if (result.getLayerItemsSize()!=0)
					addLayersOnMap(result.getLayerItems(), toTop);
			}

			public void onFailure(Throwable caught) {
				waitMessage(false);
				Window.alert("The layers doesn't exist");
			}
		});
	}

	private void addLayersByLayerItems(List<LayerItem> layerItems, final boolean onTop) {
		waitMessage(true);
		gisViewerServiceAsync.getLayersInfoByLayerItems(layerItems, new AsyncCallback<LayerItemsResult>() {			
			public void onSuccess(LayerItemsResult result) {
				waitMessage(false);
				if (!result.getStatus().equals(""))
					MessageBox.alert("Error", result.getStatus(), null);

				if (result.getLayerItemsSize()!=0)
					addLayersOnMap(result.getLayerItems(), onTop);
			}

			public void onFailure(Throwable caught) {
				waitMessage(false);
				Window.alert("The layers doesn't exist");
			}
		});
	}
	
	public void addLayerByWms(String title, String layerName, String url, boolean isBase, boolean displayInLayerSwitcher) {
		LayerItem l = new LayerItem();
		l.setBaseLayer(isBase);
		l.setTitle(title);
		l.setName(layerName);
		l.setLayer(layerName);
		l.setUrl(url);		
		l.setGeoserverUrl(url);
		l.setGeoserverWmsUrl(url);
		l.setExternal(true);
		l.setOpacity(1d);
		List<LayerItem> layerItems = new ArrayList<LayerItem>();
		layerItems.add(l);

		openLayersMap.addLayerItemByWms(l, displayInLayerSwitcher);
		layersPanel.addLayerItems(layerItems, false);
		layersPanel.updateLayersOrder();
	}

	private void addLayersOnMap(List<LayerItem> layerItems, boolean toTop) {
		for (LayerItem layerItem : layerItems)
			layerItem.setBaseLayer(false);
		openLayersMap.addLayerItems(layerItems, toTop);
		layersPanel.addLayerItems(layerItems, toTop);
		layersPanel.updateLayersOrder();
	}

	public void removeAllLayers() {
		if (openLayersMap!=null && openLayersMap.getLayerItems()!=null && openLayersMap.getLayerItems().size()>0) {
			List<LayerItem> layers = new ArrayList<LayerItem>();
			layers.addAll(openLayersMap.getLayerItems());
			for (LayerItem l: layers)
				removeLayer(l);			
		}
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.gisviewer.client.layerspanel.LayersPanelHandler#removeLayer(org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItem)
	 */
	@Override
	public void removeLayer(LayerItem layerItem) {
		openLayersMap.removeLayer(layerItem);
		layersPanel.removeLayer(layerItem);
		updateLayersOrder();
	}

	private void createOpenLayersMap() { // TODO update for pluggable layers
		openLayersMap = new OpenLayersMap(Constants.omWidth + "px",
				Constants.omHeight + "px", 12, "GisViewer ver. "+Constants.VERSION, this);

		toolBarPanel = new ToolBarPanel(this, openLayersMap);
//		toolBarPanel.setStyleName("x-toolbar");
		openLayersMap.addToolBar(toolBarPanel);
	}

	private List<String> gerUrlsForClickData(ClickDataInfo clickDataInfo) {
		List<String> urls = new ArrayList<String>();
		
		if (clickDataInfo==null)
			return urls;
		
		// get visible layers
		List<LayerItem> visibleLayerItems = layersPanel.getVisibleLayers();

		if (clickDataInfo.isPoint()) {
			// group layers by geoserver
			List<GeoserverItem> geoserverItems = groupLayersByGeoserver(visibleLayerItems, true);
	
			// create a list of url request for data (for each geoserver)
			for (GeoserverItem geoserverItem : geoserverItems) {
				String urlRequest = URLMakers.getURL(clickDataInfo, openLayersMap.getProjection(), geoserverItem.getLayerItems());
				urls.add(geoserverItem.getUrl() + urlRequest);						
			}
		} else if (clickDataInfo.isBox()) {
			for (LayerItem layerItem : visibleLayerItems)
				urls.add(URLMakers.getWfsFeatureUrl(layerItem, clickDataInfo.getBbox(), 0, "csv"));
		}
			
		return urls;
	}

	public void showDataPanel() {
		if (lastClickDataInfo==null)
			return;

		if (lastClickDataInfo.isPoint()) {
			List<String> urls = gerUrlsForClickData(lastClickDataInfo);
			mainPanel.setDataPanelWait(true);

			gisViewerServiceAsync.getDataResult(urls,
					new AsyncCallback<List<DataResult>>() {
				@Override
				public void onSuccess(List<DataResult> result) {
					DataResultPanel dpanel = new DataResultPanel();
					dpanel.setDataResult(result);
					mainPanel.setDataPanelContent(dpanel);
					mainPanel.setDataPanelWait(false);
				}
				@Override
				public void onFailure(Throwable caught) {
					mainPanel.setDataPanelWait(false);
					mainPanel.setDataPanelMessage("Data Error");
				}
			});
		} else if (lastClickDataInfo.isBox()) {
			String bbox = lastClickDataInfo.getBbox();
			mainPanel.setDataPanelWait(true);

			gisViewerServiceAsync.getDataResult(layersPanel.getVisibleLayers(), bbox, new AsyncCallback<List<WfsTable>>() {
				@Override
				public void onSuccess(List<WfsTable> result) {
					DataResultPanel dpanel = new DataResultPanel();
					dpanel.setDataResultFromWfs(result);
					mainPanel.setDataPanelContent(dpanel);
					mainPanel.setDataPanelWait(false);
				}
				@Override
				public void onFailure(Throwable caught) {
					mainPanel.setDataPanelWait(false);
					mainPanel.setDataPanelMessage("Data Error");
				}
			});
		}
	}

	private void showLegendPopup(LayerItem layerItem, final int left, final int top) {
		final Dialog dialog = new Dialog(){
			@Override
			protected void onAfterLayout() {
				super.onAfterLayout();
				if (this.getHeight()>Constants.legendDialogMaxHeight) {
					this.setAutoHeight(false);
					this.setHeight(Constants.legendDialogMaxHeight);
				}
			}
		};
		dialog.setHeading("Legend");  
		dialog.setBodyStyleName("pad-text");  
		dialog.setScrollMode(Scroll.AUTO);
		dialog.setHideOnButtonClick(true);
		dialog.setBodyStyle("background-color:#FFFFFF");
		dialog.setWidth(Constants.legendDialogWidth);
		dialog.setAutoWidth(true);
		dialog.setAutoHeight(true);
		dialog.setPosition(left, top);
		dialog.setActive(false);

		String url = layerItem.getGeoserverWmsUrl()
				+ "?service=WMS&version=1.1.1&request=GetLegendGraphic&"
				+ "layer="
				+ layerItem.getLayer()
				+ "&format=image/png"
				+ "&STYLE="
				+ layerItem.getStyle()
				+ "&LEGEND_OPTIONS=forceRule:True;dx:0.2;dy:0.2;mx:0.2;my:0.2;fontStyle:bold;"
				+ "borderColor:000000;border:true;fontColor:000000;fontSize:14";

		final Image img = new Image(url);
		img.addLoadHandler(new LoadHandler(){
			public void onLoad(LoadEvent event) {
				int h = img.getHeight();
				int w = img.getWidth();
				if (h>Constants.legendDialogMaxHeight-13) {
					dialog.setAutoHeight(false);
					dialog.setHeight(Constants.legendDialogMaxHeight);
				}
				dialog.setAutoWidth(false);
				dialog.setWidth(w<50 ? w+100 : w+50);
			}
		});
		dialog.add(img);

		dialog.show();
	}

	public void updateOpenLayersSize() {
		if (openLayersMap!=null)
			openLayersMap.updateSize();
	}

	public void openBrowserMapImage(String outputFormat, boolean isWMS) {
		String url = getMapImageUrl(outputFormat, isWMS);
		Window.open(url, "_blank", "");
	}

	@Override
	public boolean isSaveSupported() {
		return (gisViewerSaveHandler!=null);
	}


	public void saveMapImage(String outputFormat, boolean isWMS) {
		//String url = getMapImageUrl(outputFormat, isWMS);
		
		String fileName = "MapImage_"+DateTimeFormat.getFormat("yyyy-MM-dd").format(new Date());
		
		MapViewInfo mapViewInfo = openLayersMap.getMapViewInfo();
		
		String bbox = mapViewInfo.getLowerLeftX()
				+ "," + mapViewInfo.getLowerLeftY()
				+ "," + mapViewInfo.getUpperRightX()
				+ "," + mapViewInfo.getUpperRightY();
		String width = ""+mapViewInfo.getWidth();
		String height = ""+mapViewInfo.getHeight();

		// get visible layers
		List<LayerItem> visibleLayers = new ArrayList<LayerItem>();
		visibleLayers.add(openLayersMap.getActiveBaseLayer());
		visibleLayers.addAll(layersPanel.getVisibleLayers());
		List<GeoserverItem> geoserverItems = groupLayersByGeoserver(visibleLayers, false);

		ArrayList<String> geoservers = new ArrayList<String>();
		ArrayList<String> layers = new ArrayList<String>();
		ArrayList<String> styles = new ArrayList<String>();
		ArrayList<String> opacities = new ArrayList<String>();
		ArrayList<String> cqlfilters = new ArrayList<String>();
		ArrayList<String> gsrefs = new ArrayList<String>();
		

		int gsIndex=0;
		for (GeoserverItem geoserverItem: geoserverItems) {
			geoservers.add(geoserverItem.getUrl());
			//geoservers.add(URLMakers.encodeUrl(geoserverItem.getUrl()));

			for (LayerItem layerItem: geoserverItem.getLayerItems()) {
				String cqlFilter = layerItem.getCqlFilter();
				
				layers.add(layerItem.getLayer());
				styles.add((layerItem.getStyle()==null ? "null" : layerItem.getStyle()));
				opacities.add(""+layerItem.getOpacity());
				cqlfilters.add((cqlFilter.equals("") ? "null" : cqlFilter));

				gsrefs.add(""+gsIndex);
			}
		}

		gisViewerSaveHandler.saveMapImage(fileName, outputFormat, bbox, width, height, 
				geoservers.toArray(new String[geoservers.size()]), 
				layers.toArray(new String[layers.size()]), 
				styles.toArray(new String[styles.size()]), 
				opacities.toArray(new String[opacities.size()]), 
				cqlfilters.toArray(new String[cqlfilters.size()]), 
				gsrefs.toArray(new String[gsrefs.size()]));
	}

	private String getMapImageUrl(String outputFormat, boolean isWMS) {
		boolean firstGs=true, firstLayer=true;

		if (openLayersMap==null || layersPanel==null)
			return null;

		// get visible layers
		List<LayerItem> visibleLayers = new ArrayList<LayerItem>();
		visibleLayers.add(openLayersMap.getActiveBaseLayer());
		visibleLayers.addAll(layersPanel.getVisibleLayers());
		List<GeoserverItem> geoserverItems = groupLayersByGeoserver(visibleLayers, false);

		String strGeoservers="", strOpacities="", strLayers="", strStyles="", strCqlFilters="", strGeoserverRefs="";

		int gsIndex=0;
		for (GeoserverItem geoserverItem: geoserverItems) {
			String geoserverUrl = URLMakers.encodeUrl(geoserverItem.getUrl());			
			strGeoservers += ((firstGs) ? "" : ";") + geoserverUrl;			

			for (LayerItem layerItem: geoserverItem.getLayerItems()) {
				String cqlFilter = layerItem.getCqlFilter();
				strLayers += ((firstLayer) ? "" : ";") + layerItem.getLayer();
				strStyles += ((firstLayer) ? "" : ";") + (layerItem.getStyle()==null ? "null" : layerItem.getStyle());
				strOpacities += ((firstLayer) ? "" : ";") + layerItem.getOpacity();
				strCqlFilters += ((firstLayer) ? "" : ";") + (cqlFilter.equals("") ? "null" : cqlFilter);
				strGeoserverRefs += ((firstLayer) ? "" : ";") + gsIndex;

				if (firstLayer) firstLayer = false;
			}
			gsIndex++;
			if (firstGs) firstGs = false;
		}

		MapViewInfo mapViewInfo = openLayersMap.getMapViewInfo();
		String urlReturn = GWT.getModuleBaseURL() + "MapGenerator?outputFormat="+outputFormat
				+ "&bbox=" + mapViewInfo.getLowerLeftX()
				+ "," + mapViewInfo.getLowerLeftY()
				+ "," + mapViewInfo.getUpperRightX()
				+ "," + mapViewInfo.getUpperRightY()
				+ "&width=" + mapViewInfo.getWidth() 
				+ "&height=" + mapViewInfo.getHeight()
				+ "&geoservers=" + strGeoservers
				+ "&layers=" + strLayers
				+ "&styles=" + strStyles
				+ "&opacities=" + strOpacities
				+ "&cqlfilters=" + strCqlFilters
				+ "&gsrefs=" + strGeoserverRefs;

		return urlReturn;
	}



	@Override
	public void exportData(boolean force) {
		if (force) {
			List<String> urls = gerUrlsForClickData(lastClickDataInfo);
			for (String url : urls) {
				System.out.println("WFS: "+url);
				Window.open(url, "_blank", "");
			}
		} else {
			if (lastClickDataInfo!=null && lastClickDataInfo.isHardQuery())
				MessageBox.confirm("Warning"
					, ALERT_HARD_SPATIAL_QUERY
					,new Listener<MessageBoxEvent>(){
						@Override
						public void handleEvent(MessageBoxEvent be) {
							if (be.getButtonClicked().getText().contentEquals("Yes"))
								exportData(true);
						}
					}
				);
			else
				exportData(true);
		}
	}

	private String getLayerImageUrl(LayerItem layerItem, String format,
			boolean isWMS) {
		String url;
		if (isWMS) {
			String cqlFilter = layerItem.getCqlFilter();
			MapViewInfo mapViewInfo = openLayersMap.getMapViewInfo();
			url = "/wms?SERVICE=WMS&version=1.1.0&REQUEST=GetMap&LAYERS="
					+ layerItem.getLayer()
					+ "&STYLES="
					+ layerItem.getStyle()
					+ "&BBOX=" + mapViewInfo.getLowerLeftX()
					+ "," + mapViewInfo.getLowerLeftY()
					+ "," + mapViewInfo.getUpperRightX()
					+ "," + mapViewInfo.getUpperRightY()
					+ "&WIDTH=" + mapViewInfo.getWidth()
					+"&HEIGHT=" + mapViewInfo.getHeight()
					+ (cqlFilter.equals("") ? "" : "&CQL_FILTER="+cqlFilter)
					+"&SRS=EPSG:4326"
					+ "&FORMAT=" + format;
		} else {

			url = "/ows?service=WFS&version=1.0.0&request=GetFeature&typeName="
					+ layerItem.getLayer() + "&outputFormat=" + format;
		}
		return layerItem.getGeoserverUrl()+url;
	}

	@Override
	public void openBrowserLayerImage(LayerItem layerItem, String format, boolean isWMS) {
		String url = getLayerImageUrl(layerItem, format, isWMS);
		Constants.log("open layer: " + url);
		Window.open(url, "_blank", "");
	}

	@Override
	public void saveLayerImage(LayerItem layerItem, String format, boolean isWMS) {
		String url = getLayerImageUrl(layerItem, format, isWMS);
		Constants.log("open layer: " + url);
		gisViewerSaveHandler.saveLayerImage(layerItem.getName(), format, url);
	}

	@Override
	public void showLegend(LayerItem layerItem, int left, int top) {
		showLegendPopup(layerItem, left, top);
	}

	@Override
	public void changeLegend(LayerItem layerItem, String style) {
		openLayersMap.setNewStyle(layerItem, style);
	}

	@Override
	public void setOpacityLayer(LayerItem layerItem, double value) {
		openLayersMap.setOpacity(layerItem, value);
	}

	@Override
	public void showLayer(LayerItem layerItem) {
		openLayersMap.setVisibility(layerItem, true);
		layersPanel.setLayerVisible(layerItem, true);
	}

	@Override
	public void hideLayer(LayerItem layerItem) {
		openLayersMap.setVisibility(layerItem, false);
		layersPanel.setLayerVisible(layerItem, false);
	}

	@Override
	public boolean showFilterQuery(final LayerItem layerItem, int left, int top) {
		cqlFilterPanel.open(layerItem, left, top);
		return true;
	}

	@Override
	public void removeFilterQuery(LayerItem layerItem) {
		removeCqlFilter(layerItem);
	}

	private List<GeoserverItem> groupLayersByGeoserver(List<LayerItem> layerItems, boolean forData) {
		List<GeoserverItem> geoserverItems = new ArrayList<GeoserverItem>();
		boolean found;

		for (LayerItem layerItem : layerItems) {
			// FIXME: now the "fifao_*" layers were not considerated beacause they belong to fao datastore
			if (forData && (layerItem.isExternal() || layerItem.getName().substring(0, 6).equals(Constants.FAO_DATA_STORE_PREFIX)))
				; // do nothing, the layer is skipped
			else {
				found = false;
				for (GeoserverItem geoserverItem : geoserverItems)
					if (layerItem.getGeoserverWmsUrl().equals(geoserverItem.getUrl())) {
						geoserverItem.addLayerItem(layerItem);
						found = true;
						break;
					}

				if (!found) {				
					GeoserverItem geoserverItem = (forData ? 
						new GeoserverItem(layerItem.getGeoserverUrl()) : 
						new GeoserverItem(layerItem.getGeoserverWmsUrl())
					);
					geoserverItem.addLayerItem(layerItem);
					geoserverItems.add(geoserverItem);
				}
			}
		}

		return geoserverItems;
	}

	public void addGroup(String groupName) {
		this.addGroupByGroupName(groupName);
	}

	@Override
	public void updateLayersOrder() {
		this.openLayersMap.updateLayersOrder();
	}

	@Override
	public void setCQLFilter(LayerItem layerItem, String filter) {
		if (filter==null || filter.equals(""))
			removeCqlFilter(layerItem);
		else {
			openLayersMap.setCqlFilter(layerItem, filter);
			layersPanel.setCqlTip(layerItem, true);
		}
	}

	@Override
	public void removeCqlFilter(LayerItem layerItem) {
		if (layerItem.getCqlFilter()!=null || layerItem.getCqlFilter().equals("")) {
			openLayersMap.removeCqlFilter(layerItem);
			layersPanel.setCqlTip(layerItem, false);
		}
	}

	@Override
	public void activateTransect(LayerItem layerItem, String table, String field) {
		toolBarPanel.setAllUp();
		toolBarPanel.setTransectPanelVisible(layerItem, true);
		layersPanel.setTransectTip(layerItem, true);
		openLayersMap.activateTransectDraw(true);
		openLayersMap.activateZoomIn(false);
		openLayersMap.activatePan(false);
		openLayersMap.activateClickData(false);
		openLayersMap.setTableTransect(table);
		openLayersMap.setFieldTransect(field);
	}

	@Override
	public void deactivateTransect(LayerItem layerItem) {
		layersPanel.setTransectTip(layerItem, false);
		toolBarPanel.setTransectPanelVisible(layerItem, false);
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.gisviewer.client.datafeature.DataPanelHandler#panelOpen(boolean)
	 */
	@Override
	public void dataPanelOpen(boolean isOpen, int panelHeight) {
		DataPanelOpenListener handler = parameters.getDataPanelOpenHandler();
		if (handler!=null)
			handler.dataPanelOpen(isOpen, panelHeight);
	}

	/* (non-Javadoc)
	 * @see org.gcube.portlets.user.gisviewer.client.openlayers.OpenLayersHandler#selectBox(java.lang.String)
	 */
	@Override
	public void selectBox(double x1, double y1, double x2, double y2) {
		if (toolBarPanel!=null) {// && toolBarPanel.isClickBox()) {
			// create a new click data information
			lastClickDataInfo = new ClickDataInfo(x1, y1, x2, y2);

			// create a list of url request for data (for each geoserver)
			showDataPanel();
		}
	}

}
